/*
 This file is part of the digitalSTROM ds485-core library.

 Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland

 digitalSTROM ds485-core library is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation version 2
 of the License.

 digitalSTROM ds485-core library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with digitalSTROM ds485-core library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * @file
 * Helper functions to provide blocking calls to the qp framework
 *
 */

#include <semaphore.h>
#include "callqueue.h"

int ccCreate( CcInst_t *inst, QEvent const *sto[], int stoLen )
{
    QEQueue_init( &inst->queue, sto, stoLen );
    return  sem_init( &inst->sem, 0, 0 );
}

int ccFree( CcInst_t *inst )
{
    return  sem_destroy( &inst->sem );
}

QEvent* ccPostRequest( CcInst_t *inst, int signal )
{
    DS485_QueueRequestEvt   *req    = Q_NEW(DS485_QueueRequestEvt,signal);
    req->requester  = &inst->queue;
    req->sem        = &inst->sem;
    return  (QEvent*)req;
}

const QEvent* ccWaitResponse( CcInst_t *inst )
{
    sem_wait( &inst->sem );
    return QEQueue_get(&inst->queue);
}

void ccSendResponse( const QEvent* req, QEvent* res )
{
    DS485_QueueRequestEvt   *reqM   = (DS485_QueueRequestEvt*)req;
    QEQueue_postFIFO( reqM->requester, res );
    sem_post(reqM->sem);
}

